@@ -102,6 +102,10 @@ convert_df <- function(z, types, date_conv, datetime_conv, hms_conv, as_characte
 #' unnecessary to update dimensions when working with files whose sizes change
 #' frequently.
 #'
+#' The function to apply numeric formats was not extensively tested for numeric
+#' equality with spreadsheet software. There might be differences and the function
+#' has limited support for builtin styles.
+#'
 #' @seealso [wb_get_named_regions()], \link[openxlsx2:openxlsx2-package]{openxlsx2}
 #'
 #' @param file An xlsx file, [wbWorkbook] object or URL to xlsx file.
@@ -126,15 +130,14 @@ convert_df <- function(z, types, date_conv, datetime_conv, hms_conv, as_characte
 #'   If no sheet is selected, the first appearance will be selected. See [wb_get_named_regions()]
 #' @param types A named numeric indicating, the type of the data.
 #'   Names must match the returned data. See **Details** for more.
-#' @param na.strings A character vector of strings which are to be interpreted as `NA`.
-#'   Blank cells will be returned as `NA`.
-#' @param na.numbers A numeric vector of digits which are to be interpreted as `NA`.
-#'   Blank cells will be returned as `NA`.
+#' @param na Defines values to be treated as NA. Can be a character vector of strings
+#' or a named list: list(strings = ..., numbers = ...). Blank cells are always converted to `NA`.
 #' @param fill_merged_cells If `TRUE`, the value in a merged cell is given to all cells within the merge.
 #' @param keep_attributes If `TRUE` additional attributes are returned.
 #'   (These are used internally to define a cell type.)
 #' @param check_names If `TRUE` then the names of the variables in the data frame are checked to ensure that they are syntactically valid variable names.
 #' @param show_hyperlinks If `TRUE` instead of the displayed text, hyperlink targets are shown.
+#' @param apply_numfmts If `TRUE` numeric formats are applied if detected.
 #' @param ... additional arguments
 #'
 #' @examples
@@ -183,7 +186,7 @@ convert_df <- function(z, types, date_conv, datetime_conv, hms_conv, as_characte
 #' wb_to_df(wb1, start_row = 5, col_names = FALSE)
 #'
 #' # na string
-#' wb_to_df(wb1, na.strings = "a")
+#' wb_to_df(wb1, na = "a")
 #'
 #' # read names from row two and data starting from row 4
 #' wb_to_df(wb1, dims = "B2:C2,B4:C+")
@@ -219,8 +222,7 @@ wb_to_df <- function(
     rows              = NULL,
     cols              = NULL,
     detect_dates      = TRUE,
-    na.strings        = "#N/A",
-    na.numbers        = NA,
+    na                = "#N/A",
     fill_merged_cells = FALSE,
     dims,
     show_formula      = FALSE,
@@ -230,13 +232,41 @@ wb_to_df <- function(
     keep_attributes   = FALSE,
     check_names       = FALSE,
     show_hyperlinks   = FALSE,
+    apply_numfmts     = FALSE,
     ...
 ) {
 
+  arguments <- c(ls(), "na.strings", "na.numbers", "xlsx_file")
+  standardize_case_names(..., arguments = arguments)
 
-  xlsx_file <- list(...)$xlsx_file
-  standardize_case_names(...)
+  na_strings <- NULL
+  na_numbers <- NA
+  if (is.character(na)) {
+    na_strings <- na
+  }
+  if (is.list(na)) {
+    na_strings <- na$strings
+    na_numbers <- na$numbers %||% NA
+  }
 
+  if (apply_numfmts) convert <- FALSE
+
+  args <- list(...)
+  if (any(c("na.strings", "na.numbers") %in% names(args))) {
+    # # Its a little premature to activate this
+    # if (getOption("openxlsx2.soon_deprecated", default = FALSE)) {
+    #   msg <- paste0(
+    #     "na.strings and na.numbers should be combined in a named list,",
+    #     "`wb_to_df(na = list(strings = '#N/A', numbers = 999)`"
+    #   )
+    #   warning(msg, call. = FALSE)
+    # }
+
+    if ("na.strings" %in% names(args)) na_strings <- args[["na.strings"]]
+    if ("na.numbers" %in% names(args)) na_numbers <- args[["na.numbers"]]
+  }
+
+  xlsx_file <- args$xlsx_file
   if (!is.null(xlsx_file)) {
     .Deprecated(old = "xlsx_file", new = "file", package = "openxlsx2")
     file <- xlsx_file %||% file
@@ -428,7 +458,7 @@ wb_to_df <- function(
     tt <- tt[, sel, drop = FALSE]
   }
 
-  keep_rows <- keep_rows[keep_rows %in% rnams]
+  keep_rows <- intersect(keep_rows, rnams)
 
   # reduce data to selected cases only
   if (has_dims && length(keep_rows) && length(keep_cols))
@@ -466,8 +496,8 @@ wb_to_df <- function(
 
   has_na_string <- FALSE
   # convert missings
-  if (!all(is.na(na.strings))) {
-    sel <- cc$val %in% na.strings
+  if (!all(is.na(na_strings))) {
+    sel <- cc$val %in% na_strings
     if (any(sel)) {
       cc$val[sel] <- NA_character_
       cc$typ[sel] <- -1L
@@ -478,9 +508,9 @@ wb_to_df <- function(
   has_na_number <- FALSE
   # convert missings.
   # at this stage we only have characters.
-  na.numbers <- as.character(na.numbers)
-  if (!all(is.na(na.numbers))) {
-    sel <- cc$v %in% na.numbers
+  na_numbers <- as.character(na_numbers)
+  if (!all(is.na(na_numbers))) {
+    sel <- cc$v %in% na_numbers
     if (any(sel)) {
       cc$val[sel] <- NA_character_
       cc$typ[sel] <- -2L
@@ -504,7 +534,7 @@ wb_to_df <- function(
         cc$is_string <- cc$c_t %in% strings
 
       if (any(uccs %in% xlsx_date_style)) {
-        sel <- cc$c_s %in% xlsx_date_style & !cc$is_string & cc$v != ""
+        sel <- cc$c_s %in% xlsx_date_style & !cc$is_string & cc$v != "" & cc$c_t != "e"
         if (convert)
           cc$val[sel] <- date_to_unix(cc$v[sel], origin = origin)
         else
@@ -513,7 +543,7 @@ wb_to_df <- function(
       }
 
       if (any(uccs %in% xlsx_hms_style)) {
-        sel <- cc$c_s %in% xlsx_hms_style & !cc$is_string & cc$v != ""
+        sel <- cc$c_s %in% xlsx_hms_style & !cc$is_string & cc$v != "" & cc$c_t != "e"
         if (convert) {
           # if hms is loaded, we have to avoid applying convert_hms() twice
           cc$val[sel] <- cc$v[sel]
@@ -524,7 +554,7 @@ wb_to_df <- function(
       }
 
       if (any(uccs %in% xlsx_posix_style)) {
-        sel <- cc$c_s %in% xlsx_posix_style & !cc$is_string & cc$v != ""
+        sel <- cc$c_s %in% xlsx_posix_style & !cc$is_string & cc$v != "" & cc$c_t != "e"
         if (convert)
           cc$val[sel] <- date_to_unix(cc$v[sel], origin = origin, datetime = TRUE)
         else
@@ -583,6 +613,23 @@ wb_to_df <- function(
   if (has_na_string) cc$typ[cc$typ == -1] <- NA_integer_
   if (has_na_number) cc$typ[cc$typ == -2] <- NA_integer_
 
+  if (apply_numfmts) {
+
+    cc <- get_numfmt_style(wb, cc)
+
+    # apply_numfmt expects numeric, character or date/posixct
+    sel <- cc$num_fmt != "" & cc$typ %in% c(1L, 4L) & !cc$c_t %in% c("b", "e")
+    if (any(sel)) {
+      cc$val[sel] <- apply_numfmt(as.numeric(cc$val[sel]), cc$num_fmt[sel])
+      cc$typ[sel] <- 0L
+    }
+    sel <- cc$num_fmt != "" & cc$typ %in% c(0L, 2L, 3L, 5L) & !cc$c_t %in% c("b", "e")
+    if (any(sel)) {
+      cc$val[sel] <- apply_numfmt(cc$val[sel], cc$num_fmt[sel])
+      cc$typ[sel] <- 0L
+    }
+  }
+
   # prepare to create output object z
   zz <- cc[c("val", "typ")]
   zz$cols <- NA_integer_
@@ -620,7 +667,10 @@ wb_to_df <- function(
             file            = wb,
             sheet           = sheet,
             dims            = filler,
-            na.strings      = na.strings,
+            na              = list(
+              strings = na_strings,
+              numbers = na_numbers
+            ),
             convert         = FALSE,
             col_names       = FALSE,
             detect_dates    = detect_dates,
@@ -829,8 +879,7 @@ read_xlsx <- function(
   cols              = NULL,
   detect_dates      = TRUE,
   named_region,
-  na.strings        = "#N/A",
-  na.numbers        = NA,
+  na                = "#N/A",
   fill_merged_cells = FALSE,
   check_names       = FALSE,
   show_hyperlinks   = FALSE,
@@ -858,8 +907,7 @@ read_xlsx <- function(
     cols              = cols,
     detect_dates      = detect_dates,
     named_region      = named_region,
-    na.strings        = na.strings,
-    na.numbers        = na.numbers,
+    na                = na,
     fill_merged_cells = fill_merged_cells,
     check_names       = check_names,
     show_hyperlinks   = show_hyperlinks,
@@ -883,8 +931,7 @@ wb_read <- function(
   cols            = NULL,
   detect_dates    = TRUE,
   named_region,
-  na.strings      = "NA",
-  na.numbers      = NA,
+  na              = "NA",
   check_names     = FALSE,
   show_hyperlinks = FALSE,
   ...
@@ -911,8 +958,7 @@ wb_read <- function(
     cols            = cols,
     detect_dates    = detect_dates,
     named_region    = named_region,
-    na.strings      = na.strings,
-    na.numbers      = na.numbers,
+    na              = na,
     check_names     = check_names,
     show_hyperlinks = show_hyperlinks,
     ...             = ...
