@@ -69,23 +69,19 @@ dims_to_dataframe <- function(dims, fill = FALSE, empty_rm = FALSE, cc = NULL) {
   if (inherits(dims, "data.frame"))
     dims <- unlist(dims)
 
-  if (any(grepl("$", dims))) {
-    dims <- gsub("\\$", "", dims)
+  if (any(grepl("$", dims, fixed = TRUE))) {
+    dims <- gsub("$", "", dims, fixed = TRUE)
   }
 
   has_dim_sep <- FALSE
-  if (any(grepl(";", dims))) {
-    dims <- unlist(strsplit(dims, ";"))
-    has_dim_sep <- TRUE
-  }
-  if (any(grepl(",", dims))) {
-    dims <- unlist(strsplit(dims, ","))
+  if (any(grepl(",|;", dims))) {
+    dims <- unlist(strsplit(dims, split = "[,;]"))
     has_dim_sep <- TRUE
   }
 
   if (any(grep("-|\\+", dims)) && !is.null(cc)) {
-    rows <- range(as.integer(unique(cc$row_r)))
-    cols <- int2col(range(col2int(unique(cc$c_r))))
+    rows <- range(row2int(cc$row_r))
+    cols <- int2col(range(col2int(cc$c_r)))
 
     dims <- vapply(dims, function(x) expand_dims(x, cols, rows), "")
   }
@@ -99,9 +95,10 @@ dims_to_dataframe <- function(dims, fill = FALSE, empty_rm = FALSE, cc = NULL) {
   # condition 1) contains dims separator, but all dims are of
   # equal size: "A1:A5,B1:B5"
   # condition 2) either "A1:B5" or separator, but unequal size or "A1:A2,A4:A6,B1:B5"
-  if (has_dim_sep && get_dims(dims, check = TRUE)) {
+  ldims <- get_dims(dims, check = FALSE)
+  is_equal <- attr(ldims, "is_equal_sized")
 
-    ldims     <- get_dims(dims)
+  if (has_dim_sep && is_equal) {
     full_rows <- ldims$rows[[1]]
     full_cols <- ldims$cols
 
@@ -298,6 +295,7 @@ numfmt_is_posix <- function(numFmt) {
   )
   num_or_fmt <- paste0(num_fmts, collapse = "|")
   maybe_num <- grepl(pattern = num_or_fmt, x = numFmt_df$fC)
+  has_date <- grepl("y|d", numFmt_df$fC, ignore.case = TRUE)
 
   posix_fmts <- c(
     # "yy", "yyyy",
@@ -310,7 +308,7 @@ numfmt_is_posix <- function(numFmt) {
   posix_or_fmt <- paste0(posix_fmts, collapse = "|")
   maybe_posix <- grepl(pattern = posix_or_fmt, x = numFmt_df$fC)
 
-  z <- numFmt_df$numFmtId[maybe_posix & !maybe_num]
+  z <- numFmt_df$numFmtId[maybe_posix & has_date & !maybe_num]
   if (length(z) == 0) z <- NULL
   z
 }
